
import calendar
import coloredlogs
from datetime import datetime
import logging
import numpy as np
import pandas as pd
import os
from copy import deepcopy
import coloredlogs
import netCDF4 as nc
from collections import OrderedDict
from datetime import timedelta, datetime

from inicheck.tools import get_user_config, check_config, cast_all_variables
from inicheck.output import generate_config, print_config_report
from inicheck.config import MasterConfig
import snowav
from snowav.utils.utilities import masks, get_snowav_path
from snowav.utils.get_topo_stats import get_topo_stats
from snowav.framework.outputs import outputs
from snowav.database.database import connect
from snowav.database.models import AwsmInputsOutputs
from snowav.utils.wyhr import handle_year_stradling, calculate_date_from_wyhr


class UserConfig(object):
    """ Config file class.

    Args
    ------
    config_file: file path
    external_logger: logger from awsm
    awsm: awsm object
    end_date: string
    """

    def __init__(self, config_file, external_logger = None, awsm = None,
                 end_date = None):

        print('Reading {} and loading files...'.format(config_file))

        self.config_file = config_file
        snowav_mcfg = MasterConfig(modules = 'snowav')
        ucfg = get_user_config(self.config_file, mcfg=snowav_mcfg)
        ucfg.apply_recipes()
        ucfg = cast_all_variables(ucfg, ucfg.mcfg)
        self.snowav_path = get_snowav_path()

        warnings, errors = check_config(ucfg)
        if errors != [] or warnings != []:
            print_config_report(warnings, errors)

        self.tmp_log = []
        self.tmp_err = []
        self.tmp_warn = []
        self.proc_time_start = datetime.now()

        ####################################################
        #            snowav                                #
        ####################################################
        self.loglevel = ucfg.cfg['snowav']['log_level'].upper()
        self.log_to_file = ucfg.cfg['snowav']['log_to_file']
        self.save_path = ucfg.cfg['snowav']['save_path']
        self.units = ucfg.cfg['snowav']['units']
        self.elev_bins = ucfg.cfg['snowav']['elev_bins']
        self.directory = ucfg.cfg['snowav']['directory']
        self.dempath = ucfg.cfg['snowav']['dempath']
        self.run_name = ucfg.cfg['snowav']['run_name']
        self.plotorder = ucfg.cfg['snowav']['masks']
        self.plotlabels = ucfg.cfg['snowav']['plotlabels']

        ####################################################
        #           run                                    #
        ####################################################
        self.dplcs = ucfg.cfg['run']['decimals']
        self.start_date = ucfg.cfg['run']['start_date']
        self.end_date = ucfg.cfg['run']['end_date']

        if end_date is not None:
            self.end_date = end_date
            self.tmp_log.append(' Overriding config end_date with '
                                '{} given with snowav call'.format(end_date))

            if self.end_date <= self.start_date:
                raise Exception('end_date {} earlier than start_date {}'.format(
                                self.end_date, self.start_date))

        if self.start_date is not None and self.end_date is not None:
            self.start_date = self.start_date
            self.end_date = self.end_date

            if self.start_date >= self.end_date:
                self.tmp_log.append(' Error: [run] start_date >= [run] end_date')
                raise Exception('Error: [run] start_date >= [run] end_date')

        else:
            self.tmp_log.append(' [run] start_date and/or end_date was not '
                                'defined in config file, will be assigned '
                                'by available dates in directory')

        self.all_subdirs = ucfg.cfg['run']['all_subdirs']

        if (ucfg.cfg['run']['directory'] is None) and (awsm is not None):
            if self.all_subdirs is True:
                self.run_dirs = ([awsm.pathr + s for s in
                                os.listdir(awsm.pathr)
                                if (os.path.isdir(awsm.pathr + s)) ])
            else:
                self.run_dirs = awsm.pathr
                if type(self.run_dirs) != list:
                    self.run_dirs = [self.run_dirs]

        else:
            directory = ucfg.cfg['run']['directory']

            if len(directory) == 1:
                directory = directory[0]

            if self.all_subdirs is True:
                self.run_dirs = ([directory + s for s in os.listdir(directory)
                                if (os.path.isdir(directory + s))])
            else:
                self.run_dirs = ucfg.cfg['run']['directory']
                if type(self.run_dirs) != list:
                    self.run_dirs = [self.run_dirs]

        self.run_dirs.sort()

        ####################################################
        #         database
        ####################################################
        self.mysql = ucfg.cfg['database']['mysql']
        self.db_user = ucfg.cfg['database']['user']
        self.db_password = ucfg.cfg['database']['password']
        self.db_host = ucfg.cfg['database']['host']
        self.db_port = ucfg.cfg['database']['port']
        self.db_convert = ucfg.cfg['database']['convert_ws']
        self.add_basins = ucfg.cfg['database']['add_basins']
        self.db_overwrite = ucfg.cfg['database']['overwrite']
        self.properties = ucfg.cfg['database']['properties']
        self.sqlite = ucfg.cfg['database']['sqlite']

        base_bands = ['swi_z','evap_z','swe_z','depth','density',
            'coldcont', 'precip_z']

        f = False
        for band in base_bands:
            if band not in self.properties:
                self.tmp_log.append(' WARNING! Config option [database] '
                    'properties does not contain {}'.format(band))
                f = True

        if f:
            self.tmp_log.append(' WARNING! Suggest config option [database] '
                'properties contain at least {} for most functionality to '
                'run'.format(base_bands))

        if ((self.mysql is not None) and
            ((self.db_user is None) or
             (self.db_password is None) or
             (self.db_host is None) or
             (self.db_port is None)) ):
            raise Exception('If using config option [database] mysql, must '
                            'also supply user, password, host, and port')

        if self.sqlite is not None:
            if not os.path.isdir(os.path.dirname(self.sqlite)):
                raise Exception('{} does not contain a valid base '
                                'path'.format(self.sqlite))
            self.sqlite = 'sqlite:///' + self.sqlite

            if self.mysql is not None:
                raise Exception('Config option [database] section contains '
                'both "mysql" and "sqlite" entries, pick one.')

        ####################################################
        #           validate                               #
        ####################################################
        self.val_stns = ucfg.cfg['validate']['stations']
        self.val_lbls = ucfg.cfg['validate']['labels']
        self.val_client = ucfg.cfg['validate']['client']
        self.pre_val_stns = ucfg.cfg['validate']['pre_stations']
        self.pre_val_lbls = ucfg.cfg['validate']['pre_labels']
        self.wxdb_user = ucfg.cfg['validate']['user']
        self.wxdb_password = ucfg.cfg['validate']['password']
        self.wxdb_host = ucfg.cfg['validate']['host']
        self.wxdb_port = ucfg.cfg['validate']['port']
        self.point_values = ucfg.cfg['validate']['point_values']
        self.point_values_csv = ucfg.cfg['validate']['point_values_csv']
        self.point_values_date = ucfg.cfg['validate']['point_values_date']
        self.point_values_properties = ucfg.cfg['validate']['point_values_properties']
        self.point_values_heading = ucfg.cfg['validate']['point_values_heading']
        self.point_values_settings = ucfg.cfg['validate']['point_values_settings']

        if len(self.point_values_settings) != 14:
            self.tmp_log.append(' Expected [validate] point_values_settings to '
                                'have 14 values, point_values being set to False')
            self.point_values = False

        for n in range(0,10):
            self.point_values_settings[n] = int(self.point_values_settings[n])

        if type(self.point_values_properties) != list:
            self.point_values_properties = [self.point_values_properties]

        if type(self.point_values_heading) != list:
            self.point_values_heading = [self.point_values_heading]

        if type(self.point_values_csv) != list:
            self.point_values_csv = [self.point_values_csv]

        if self.point_values and self.point_values_csv is None:
            self.point_values = False
            self.tmp_log.append(' Config option [validate] point_values_csv '
                                'was not supplied, point_values being set '
                                'to False')

        if self.point_values and self.point_values_date is None:
            self.point_values = False
            self.tmp_log.append(' Config option [validate] point_values_date '
                                'was not supplied, point_values being set '
                                'to False')

        if not (len(self.point_values_csv) == len(self.point_values_properties) ==
                len(self.point_values_heading)):
            self.tmp_log.append(' Must supply the same number of [validate] '
                                'point_values_csv, point_values_properties, '
                                'and point_values_heading values, point_values '
                                'being set to False')

        ####################################################
        #           diagnostics                            #
        ####################################################
        self.diagnostics_flag = ucfg.cfg['diagnostics']['diagnostics']
        self.diag_basins = ucfg.cfg['diagnostics']['basins']
        self.diag_limit = ucfg.cfg['diagnostics']['limit']

        if self.diagnostics_flag:
            if self.diag_basins is not None:
                for basin in self.diag_basins:
                    if basin not in self.plotorder:
                        self.tmp_log.append(' Config option [diagnostics] basin: '
                                    '{} does not match what was supplied in '
                                    '[snowav] masks: {}, diagnostics set '
                                    'to False'.format(basin, self.plotorder))
                        self.diagnostics_flag = False

        self.inputs_flag = ucfg.cfg['diagnostics']['inputs_table']
        self.inputs_variables = ucfg.cfg['diagnostics']['inputs_variables']
        self.inputs_percentiles = ucfg.cfg['diagnostics']['inputs_percentiles']
        self.inputs_methods = ucfg.cfg['diagnostics']['inputs_methods']
        self.inputs_basins = ucfg.cfg['diagnostics']['inputs_basins']

        # if self.inputs_flag:
        if self.inputs_basins is not None and self.plotorder is not None:
            for basin in self.inputs_basins:
                if basin not in self.plotorder:
                    self.tmp_log.append(' Config option [diagnostics] '
                                        'inputs_basins: {} does not match what '
                                        'was supplied in [snowav] masks: {}, '
                                        'inputs set to '
                                        'False'.format(basin, plotorder))
                    self.inputs_flag = False

        if self.inputs_flag:
            s = [x + ', ' for x in self.inputs_variables]
            self.tmp_log.append(' Using variables {} for inputs '
                                'summary'.format(''.join(s)))

            s = [x + ', ' for x in self.inputs_methods]
            self.tmp_log.append(' Using methods {} for inputs '
                                'summary'.format(''.join(s)))

        ####################################################
        #          plots                                   #
        ####################################################
        self.dpi = ucfg.cfg['plots']['dpi']
        self.depth_clip = ucfg.cfg['plots']['depth_clip']
        self.percent_min = ucfg.cfg['plots']['percent_min']
        self.percent_max = ucfg.cfg['plots']['percent_max']
        self.subs_fig = ucfg.cfg['plots']['subs_fig']
        self.density_flag = ucfg.cfg['plots']['density']
        self.swi_flag = ucfg.cfg['plots']['swi']
        self.current_image_flag = ucfg.cfg['plots']['current_image']
        self.image_change_flag = ucfg.cfg['plots']['image_change']
        self.cold_content_flag = ucfg.cfg['plots']['cold_content']
        self.swe_volume_flag = ucfg.cfg['plots']['swe_volume']
        self.basin_total_flag = ucfg.cfg['plots']['basin_total']
        self.stn_validate_flag = ucfg.cfg['plots']['stn_validate']
        self.nash_sut_flag = ucfg.cfg['plots']['disp_nash_sut']
        self.stns_file = ucfg.cfg['plots']['stns_file']
        self.precip_validate_flag = ucfg.cfg['plots']['precip_validate']
        self.inputs_fig_flag = ucfg.cfg['plots']['inputs']
        self.plots_inputs_variables = ucfg.cfg['plots']['inputs_variables']
        self.compare_runs_flag = ucfg.cfg['plots']['compare_runs']
        self.compare_run_names = ucfg.cfg['plots']['compare_run_names']
        self.compare_run_labels = ucfg.cfg['plots']['compare_run_labels']
        self.compare_run_wys = ucfg.cfg['plots']['compare_run_wys']
        self.precip_depth_flag = ucfg.cfg['plots']['precip_depth']
        self.basin_detail_flag = ucfg.cfg['plots']['basin_detail']
        self.update_file = ucfg.cfg['plots']['update_file']
        self.print_args_dict = ucfg.cfg['plots']['print_args_dict']
        self.figsize = (ucfg.cfg['plots']['fig_length'],
                        ucfg.cfg['plots']['fig_height'])
        self.write_properties = ucfg.cfg['plots']['write_properties']
        self.point_values_flag = ucfg.cfg['plots']['point_values']

        if (self.write_properties is not None and
        type(self.write_properties) != list):
            self.write_properties = [self.write_properties]

        numbers = ucfg.cfg['plots']['update_numbers']

        if numbers is not None:
            if type(numbers) != list:
                numbers = [numbers]
            self.update_numbers = [x - 1 for x in numbers]
        else:
            self.update_numbers = None

        if (self.compare_runs_flag and ((self.compare_run_names is None) or
            (self.compare_run_labels is None) or self.compare_run_wys is None)):
            self.tmp_log.append(' Config option [plots] compare_runs set to True, '
                                'but one of compare_run_names, compare_run_labels, '
                                'or compare_run_wys is empty, setting '
                                'compare_runs to False')
            self.compare_runs_flag = False

        if (self.compare_runs_flag and
            (len(self.compare_run_names) != len(self.compare_run_labels))):
            self.tmp_log.append(' Config option [plots] compare_runs set to True, '
                                'must supply equal length compare_run_names and  '
                                'compare_run_labels, resetting compare_runs to False')
            self.compare_runs_flag = False

        if self.update_file is not None:
            self.flt_flag = True
        else:
            self.flt_flag = False

        if (self.precip_validate_flag and ((self.val_client is None) or
           (self.pre_val_stns is None) or (self.pre_val_lbls is None))):
            self.tmp_log.append(' Config option [plots] precip_validate is being '
                                'set to False')

            self.precip_validate_flag = False

        if (self.stn_validate_flag and (self.val_client is None) or
           (self.val_stns is None) or (self.val_lbls is None) or
           (self.wxdb_user is None) or (self.wxdb_password is None) ):
            self.tmp_log.append(' Config option [plots] stn_validate is being '
                                'set to False')

            self.stn_validate_flag = False

        for var in self.plots_inputs_variables:
            if var not in self.inputs_variables:
                self.plots_inputs_variables.remove(var)
                self.tmp_log.append(' Config option [plots] inputs_variables '
                                    'value {} not present in [diagnostics] '
                                    'inputs_variables, being removed'.format(var))

        ####################################################
        #          report                                  #
        ####################################################
        self.report_flag = ucfg.cfg['report']['report']
        self.print_latex = ucfg.cfg['report']['print_latex']
        self.report_name = ucfg.cfg['report']['file']
        self.rep_title = ucfg.cfg['report']['title']
        self.rep_path = ucfg.cfg['report']['save_path']
        self.env_path = ucfg.cfg['report']['env_path']
        self.templ_path = ucfg.cfg['report']['templ_path']
        self.tex_file = ucfg.cfg['report']['tex_file']
        self.summary_file = ucfg.cfg['report']['summary']
        self.figs_tpl_path = ucfg.cfg['report']['figs_tpl_path']
        self.flight_figs = ucfg.cfg['report']['flight_figs']
        self.tables = ucfg.cfg['report']['tables']
        self.report_diagnostics = ucfg.cfg['report']['diagnostics']
        self.report_diagnostics_day = ucfg.cfg['report']['diagnostics_day']
        self.rep_dplcs = ucfg.cfg['report']['decimals']

        if self.report_diagnostics and (not self.inputs_fig_flag or not self.diagnostics_flag):
            self.tmp_log.append(" [report] diagnostics: True, but must also have "
                "[plots] inputs: True and [diagnostics] diagnostics: True, "
                "setting to False")
            self.report_diagnostics = False

        if self.report_diagnostics and self.report_diagnostics_day[0] != 'any':

            if calendar.day_name[datetime.now().weekday()] not in self.report_diagnostics_day:
                self.report_diagnostics = False
                self.tmp_log.append(" Per [report] diagnostics_day: {}, "
                    "setting diagnostics: False".format(self.report_diagnostics_day))

        self.rep_swi_flag = ucfg.cfg['report']['swi']
        if not self.swi_flag:
            self.rep_swi_flag = False

        self.rep_image_change_flag = ucfg.cfg['report']['image_change']
        if not self.image_change_flag:
            self.rep_image_change_flag = False

        self.rep_cold_content_flag = ucfg.cfg['report']['cold_content']
        if not self.cold_content_flag:
            self.rep_cold_content_flag = False

        self.rep_swe_volume_flag = ucfg.cfg['report']['swe_volume']
        if not self.swe_volume_flag:
            self.rep_swe_volume_flag = False

        self.rep_basin_total_flag = ucfg.cfg['report']['basin_total']
        if not self.basin_total_flag:
            self.rep_basin_total_flag = False

        self.rep_stn_validate_flag = ucfg.cfg['report']['stn_validate']
        if not self.stn_validate_flag:
            self.rep_stn_validate_flag = False

        self.rep_compare_runs_flag = ucfg.cfg['report']['compare_runs']
        if not self.compare_runs_flag:
            self.rep_compare_runs_flag = False

        self.rep_precip_depth_flag = ucfg.cfg['report']['precip_depth']
        if not self.precip_depth_flag:
            self.rep_precip_depth_flag = False

        # check paths to see if they need default snowav path
        if self.env_path is None:
            self.env_path = os.path.abspath('./snowav/report/template/section_text')
        if self.templ_path is None:
            self.templ_path = os.path.abspath('./snowav/report/template')
        if self.summary_file is None:
            self.summary_file = os.path.abspath('./snowav/report/template/section_text/report_summary.txt')
        if self.tex_file is None:
            self.tex_file = os.path.abspath('./snowav/report/template/snowav_report.tex')
        if self.figs_tpl_path is None:
            self.figs_tpl_path = os.path.abspath('./snowav/report/figs')

        ####################################################
        #           forecast                               #
        ####################################################
        self.forecast_flag = ucfg.cfg['forecast']['report']

        if self.forecast_flag:
            self.for_start_date = ucfg.cfg['forecast']['start_date']
            self.for_end_date = ucfg.cfg['forecast']['end_date']
            self.for_run_name = ucfg.cfg['forecast']['run_name']

            if self.for_start_date >= self.for_end_date:
                self.tmp_log.append(' Error: config option [forecast] start_date > '
                                    'end_date')
                raise Exception('Config option [forecast] start_date is greater '
                                'than end_date')

            self.for_run_dir = ([ucfg.cfg['forecast']['run_dir'] + s for s in
                            os.listdir(ucfg.cfg['forecast']['run_dir'])
                            if (os.path.isdir(ucfg.cfg['forecast']['run_dir'] + s)) ])

            self.for_run_dir.sort()

        ####################################################
        #           query                                  #
        ####################################################
        self.query_flag = ucfg.cfg['query']['query']
        self.q_basins = ucfg.cfg['query']['basins']
        self.q_value = ucfg.cfg['query']['value']
        self.q_run_name = ucfg.cfg['query']['run_name']
        self.q_print_all_runs = ucfg.cfg['query']['print_all_runs']
        self.q_start_date = ucfg.cfg['query']['start_date']
        self.q_end_date = ucfg.cfg['query']['end_date']
        self.q_total = ucfg.cfg['query']['total']
        self.q_output = ucfg.cfg['query']['output']
        self.q_csv_base_path = ucfg.cfg['query']['csv_base_path']
        self.q_database = ucfg.cfg['query']['database']

        ####################################################
        #           inflow                                 #
        ####################################################
        self.inflow_flag = ucfg.cfg['inflow']['inflow']
        self.inflow_data = ucfg.cfg['inflow']['inflow_data']
        self.summary_csv = ucfg.cfg['inflow']['summary_csv']
        self.inflow_headings = ucfg.cfg['inflow']['inflow_headings']
        self.basin_headings = ucfg.cfg['inflow']['basin_headings']
        self.sheet_name = ucfg.cfg['inflow']['sheet_name']
        self.skiprows = ucfg.cfg['inflow']['skiprows']
        self.overwrite = ucfg.cfg['inflow']['overwrite']
        self.file_base = ucfg.cfg['inflow']['file_base']
        self.date_idx = ucfg.cfg['inflow']['date_idx']
        self.convert = ucfg.cfg['inflow']['convert']

        self.ucfg = ucfg

    def assign_vars(self, kwargs):
        """ Assign attributes.

        Args
        ------
        kwargs {dict}: {item, value}
        """

        if not isinstance(kwargs, dict):
            raise TypeError("kwargs must be dict")

        for item, v in kwargs.items():
            setattr(self, item, v)

    def parse(self, external_logger=None):
        """ Parse config options. """

        self.snowav_version = snowav.__version__
        self.cclimit = -5*1000*1000

        self.barcolors = ['xkcd:cobalt',
                          'xkcd:mustard green',
                          'xkcd:lichen',
                          'xkcd:pale green',
                          'xkcd:blue green',
                          'xkcd:bluish purple',
                          'xkcd:lightish purple',
                          'xkcd:deep magenta',
                          'xkcd:burgundy',
                          'red']

        out = masks(self.dempath, self.db_convert, plotorder = self.plotorder,
                    plotlabels = self.plotlabels)

        self.dem = out['dem']
        self.veg_type = out['veg_type']
        self.masks = out['masks']
        self.nrows = out['nrows']
        self.ncols = out['ncols']
        self.plotorder = out['plotorder']
        self.labels = out['labels']

        for log in out['logger']:
            self.tmp_log.append(log)

        # Establish database connection
        self.basins, cnx, out = connect(sqlite = self.sqlite, sql = self.mysql,
                                   plotorder = self.plotorder, user = self.db_user,
                                   password = self.db_password, host = self.db_host,
                                   port = self.db_port, convert = self.db_convert,
                                   add = self.add_basins)
        self.connector = cnx

        for log in out:
            self.tmp_log.append(log)

        if self.loglevel == 'DEBUG':
            for basin in self.basins:
                self.tmp_log.append(' {}: {}'.format(basin, self.basins[basin]))

        # Check snow.nc file location, get topo stats and water year
        sfile = os.path.join(self.run_dirs[0],'snow.nc')

        if os.path.isfile(sfile):
            topo = get_topo_stats(sfile)
            self.snow_x = topo['x']
            self.snow_y = topo['y']
            self.pixel = int(topo['dv'])

            ncf = nc.Dataset(sfile)
            t = nc.num2date(ncf.variables['time'][0],ncf.variables['time'].units)
            ncf.close()
            self.wy = handle_year_stradling(t) + 1

        else:
            print('\nGiven config options, expecting to find:\n {}\nto load topo '
                  'stats but is not a valid file\nCheck config [run] options, see '
                  'CoreConfig.ini for details\n'.format(sfile))
            raise Exception('{} not a valid file'.format(sfile))

        # make the bins
        edges = np.arange(self.elev_bins[0],
                          self.elev_bins[1]+self.elev_bins[2],
                          self.elev_bins[2])

        # use for definition
        self.edges = np.arange(self.elev_bins[0]-self.elev_bins[2],
                               self.elev_bins[1],
                               self.elev_bins[2])

        v = self.properties
        if self.inputs_flag:
            v += self.inputs_variables

        # get variables that will be used in processing
        self.variables = AwsmInputsOutputs()
        self.variables.make_variables(v,
                                      self.edges,
                                      self.masks.keys())

        if self.units == 'TAF':
            self.conversion_factor = ((self.pixel**2)*0.000000810713194*0.001)
            self.depth_factor = 0.03937
            self.dem = self.dem * 3.28
            self.depthlbl = 'in'
            self.vollbl = self.units
            self.elevlbl = 'ft'

            if max(self.edges) < 5000:
                self.tmp_log.append(" WARNING! Config options [snowav] units: TAF "
                    "and elev_bins: {} may not match! Consider changing elev_bins "
                    "values".format(self.elev_bins))

        if self.units == "SI":
            self.conversion_factor = ((self.pixel**2)*0.000000810713194)*1233.48/1e9
            self.depth_factor = 0.01
            self.depthlbl = 'cm'
            self.vollbl = 'M$M^3$'
            self.elevlbl = 'm'

            if max(self.edges) > 5000:
                self.tmp_log.append(" WARNING! Config options [snowav] units: SI "
                    "and elev_bins: {} may not match! Consider changing elev_bins "
                    "values".format(self.elev_bins))

        self.ixd = np.digitize(self.dem,edges)
        self.xlims = (0,len(edges))

        if self.loglevel == 'DEBUG' and self.log_to_file is not True:
            print('Reading files in {}...'.format(self.run_dirs[0].split('runs')[0]))

        results = outputs(self.run_dirs, self.wy, self.properties,
                    self.start_date, self.end_date, None, self.loglevel)

        out = results['outputs']
        all_dirs = results['dirs']
        dirs = results['run_dirs']
        rdict = results['rdict']
        log = results['log']

        # If there was an error parsing files catch and log it
        if out == [] and all_dirs == [] and 'not a valid file' in log[-1]:

            self.tmp_log.append(log[-1])
            if self.start_date is not None and self.end_date is not None:
                ext_shr = (self.directory +
                          '_'  +
                          self.start_date.date().strftime("%Y%m%d") +
                          '_' +
                          self.end_date.date().strftime("%Y%m%d") )
                self.figs_path = os.path.join(self.save_path, '{}/'.format(ext_shr))

                if external_logger == None:
                    createLog(self)
                else:
                    self._logger = external_logger

            raise Exception(log[-1])

        for l in log:
            self.tmp_log.append(l)

        if out['dates'] == []:
            raise Exception('Supplied [run] directory, start_date, and end_date '
                  'give no valid snow files')

        self.outputs = out
        self.run_dirs = dirs
        self.all_dirs = all_dirs
        self.rundirs_dict = rdict
        self.all_dirs_flt = deepcopy(all_dirs)

        if self.start_date is not None and self.end_date is None:
            self.end_date = self.outputs['dates'][-1]
            self.tmp_log.append(' Config options [run] end_date '
                                'not specified, assigning '
                                '{} and {}'.format(self.start_date,self.end_date))

            self.ixs = 0
            self.ixe = len(self.outputs['dates']) - 1

        # Otherwise, get closest dates and make indices
        else:
            self.start_date = self.outputs['dates'][0]
            self.end_date = self.outputs['dates'][-1]
            self.ixs = 0
            self.ixe = len(self.outputs['dates']) - 1

        if ((self.start_date.date() < self.outputs['dates'][0].date())
            or (self.end_date.date() > self.outputs['dates'][-1].date())):
            raise Exception('ERROR! Config option [run] start_date or end_date '
                            'outside of date range found in [run] directory')

        # Since model outputs at 23:00, step the figure and report dates to
        # show 00:00 the next day (unless start of water year)
        if self.start_date == datetime(self.wy-1,10,1,23,0,0):
            self.report_start = self.start_date

        else:
            self.report_start = self.start_date + timedelta(hours=1)

        # Copy the config file where figs will be saved
        # use directory if only plotting figures from database and don't
        # have start_date, end_date
        extf = os.path.splitext(os.path.split(self.config_file)[1])
        ext_shr = (self.directory +
                  '_'  +
                  self.start_date.date().strftime("%Y%m%d") +
                  '_' +
                  self.end_date.date().strftime("%Y%m%d") )
        self.figs_path = os.path.join(self.save_path, '{}/'.format(ext_shr))

        # get forecast outputs
        if self.forecast_flag:

            results = outputs(self.for_run_dirs, self.wy, self.properties,
                None, None, None, self.loglevel)

            self.for_outputs = results['outputs']
            self.for_run_dirs = results['run_dirs']
            self.for_rundirs_dict = results['rdict']
            self.for_ixs = 0
            self.for_ixe = len(self.for_outputs['swe_z']) - 1

        # Get outputs for flights
        if self.flt_flag:

            file = self.update_file
            p = nc.Dataset(file, 'r')

            if self.update_numbers is None:
                times = p.variables['time'][:]
            else:
                if sum([x > len(p.variables['time']) for x in self.update_numbers]) > 0:
                    self.tmp_log.append(' Value in [plots] update_numbers out of '
                                        'range, max is {}, flight update figs '
                                        'being set to False'.format(len(p.variables['time'])))
                    times = []
                    self.flt_flag = False

                else:
                    times = p.variables['time'][self.update_numbers]

            p.close()

            flight_dates = []
            pre_flight_dates = []

            for time in times:
                wydate = calculate_date_from_wyhr(int(time), self.wy)
                pre_wydate = calculate_date_from_wyhr(int(time-24), self.wy)
                flight_dates = np.append(flight_dates,wydate)
                pre_flight_dates = np.append(pre_flight_dates,pre_wydate)

            if self.loglevel == 'DEBUG' and self.log_to_file is not True:
                print('Reading files in {} for flight updates'
                      '...'.format(self.run_dirs[0].split('runs')[0]))

            results = outputs(self.all_dirs_flt, self.wy, self.properties,
                None, None, flight_dates, self.loglevel)

            self.flight_outputs = results['outputs']
            self.run_dirs_flt = results['run_dirs']
            self.flt_rundirs_dict = results['rdict']
            self.flight_diff_dates = results['outputs']['dates']
            self.pre_flight_outputs = results['outputs']

            results = outputs(self.all_dirs_flt, self.wy, self.properties,
                None, None, pre_flight_dates, self.loglevel)

            self.pre_flight_outputs = results['outputs']

            # If there are no flights in the period, set to false for the flight
            # difference figure and report
            if not self.run_dirs_flt:
                self.flt_flag = False
                self.tmp_log.append(' Config option [plots] update_file was '
                                    'supplied, but no snow.nc files were found in '
                                    '[run] directory that fit the date range, no '
                                    'flight difference figure will be made')

        self.report_date = self.end_date + timedelta(hours=1)
        parts = self.report_name.split('.')
        self.report_name = (parts[0] + self.report_date.date().strftime("%Y%m%d") +
                           '.' + parts[1])

        if not os.path.exists(self.figs_path):
            os.makedirs(self.figs_path)

        config_copy = '{}{}{}'.format(self.figs_path, ext_shr, extf[1])
        generate_config(self.ucfg, config_copy)

        if external_logger == None:
            createLog(self)
        else:
            self._logger = external_logger

        if self.inputs_basins is None:
            self.inputs_basins = [self.plotorder[0]]


def createLog(self):
    '''
    Create log file and print out saved logging statements.
    '''

    level_styles = {'info': {'color': 'white'},
                    'notice': {'color': 'magenta'},
                    'verbose': {'color': 'blue'},
                    'success': {'color': 'green', 'bold': True},
                    'spam': {'color': 'green', 'faint': True},
                    'critical': {'color': 'red', 'bold': True},
                    'error': {'color': 'red'},
                    'debug': {'color': 'green'},
                    'warning': {'color': 'yellow'}}

    field_styles =  {'hostname': {'color': 'magenta'},
                     'programname': {'color': 'cyan'},
                     'name': {'color': 'white'},
                     'levelname': {'color': 'white', 'bold': True},
                     'asctime': {'color': 'green'}}

    # start logging
    loglevel = self.loglevel
    numeric_level = getattr(logging, loglevel, None)

    if not isinstance(numeric_level, int):
        raise ValueError('Invalid log level: %s' % loglevel)

    # setup the logging
    logfile = None
    if self.log_to_file:
        logfile = os.path.join(self.figs_path, 'log_snowav.out')
        # let user know
        print('Logging to file: {}'.format(logfile))

    fmt = '%(levelname)s:%(module)s:%(message)s'

    if logfile is not None:

        for handler in logging.root.handlers[:]:
            logging.root.removeHandler(handler)

        logging.basicConfig(filename=logfile,
                            filemode='w',
                            level=numeric_level,
                            format=fmt)
    else:
        logging.basicConfig(level=numeric_level)
        coloredlogs.install(level=numeric_level,
                            fmt=fmt,
                            level_styles=level_styles,
                            field_styles=field_styles)

    self._loglevel = numeric_level

    self._logger = logging.getLogger(__name__)

    if len(self.tmp_log) > 0:
        for l in self.tmp_log:
            self._logger.info(l)
    if len(self.tmp_warn) > 0:
        for l in self.tmp_warn:
            self._logger.warning(l)
    if len(self.tmp_err) > 0:
        for l in self.tmp_err:
            self._logger.error(l)




import numpy as np
import pandas as pd
from snowav.database.database import insert, query
from datetime import datetime
from snowav.database.tables import RunMetadata, Watershed, Basin, Results, VariableUnits

def package(connector, lbls, basins, df, run_id, vid, output, dtime, run_name):
    '''
    Put process() results on the database.

    Args
    ------
    connector : str
        database connector
    lbls : dict
    basins : dict
    run_id : int
    vid : int
        variable id
    df : DataFrame
    output : str
        output variable (i.e. 'swe_z')
    dtime: datetime

    '''

    if ('z' in output) or ('depth' in output):
        lbl = lbls['depthlbl']
    if ('vol' in output) or ('avail' in output):
        lbl = lbls['vollbl']
    if output == 'density':
        lbl = 'kg/m^3'
    if output == 'coldcont':
        lbl = 'MJ'
    if output == 'snow_line':
        lbl = 'ft'

    for basin in df:

        for iters, val in enumerate(df[basin].values):
            if np.isnan(val):
                val = None

            else:
                val = float(val)

            values = {'basin_id': int(basins[basin]['basin_id']),
                      'run_id': int(run_id),
                      'date_time': dtime,
                      'variable': output,
                      'variable_id': int(vid[output]),
                      'value': val,
                      'elevation': str(df[basin].index[iters])}

            insert(connector,'Results',values)




import numpy as np
from matplotlib import pyplot as plt
import matplotlib.colors
from mpl_toolkits.axes_grid1 import make_axes_locatable
import seaborn as sns
import utm
import pandas as pd
import cmocean
import snowav.framework.figures
from mpl_toolkits.axes_grid1.inset_locator import inset_axes
import copy

def point_values(output, value, df, imgxy, filename, dem, figs_path, veg_type,
                 heading, model_date, course_date, settings, pixel, logger):
    '''
    Read in specified csv file of snow course validation locations, write out
    model values at those points, and create figures.

    Args
    ------
    output : array
        array of output values
    value : str
    df : DataFrame
        from point_values_csv file:
        name        latitude    longitude
        Bishop Pass 37.1        -118.557
        Bench Lake  36.958      -118.445

    imgxy : array
    x and y coordinates for the image, generated from get_topo_stats.py
    filename : str
    dem : array
    figs_path : str
    veg_type : array
    heading : str
    model_date : str
    course_date : str
    settings : list
    pixel : int
    logger

    '''

    if value == 'swe_z':
        header = cbarlabel = 'SWE [in]'

    if value == 'density':
        header = 'density [kg/m^3]'
        cbarlabel = r'density [$kg/m^3$]'

    if value == 'depth':
        header = cbarlabel = 'depth [in]'

    if heading is not None:
        cols = ['station','x','y','model x','model y', header,
                '9 pixel median', heading]
    else:
        cols = ['station','x','y','model x','model y', header,
                '9 pixel median']

    pixel_swe = pd.DataFrame(columns = cols)

    # get closest model pixel
    for i in df.index:
        sta = df.loc[i,'name']
        lat = df.loc[i,'latitude']
        lon = df.loc[i,'longitude']

        ll = utm.from_latlon(lat,lon)
        xind = np.where(abs(imgxy[0]-ll[0]) == min(abs(imgxy[0]-ll[0])))[0]
        yind = np.where(abs(imgxy[1]-ll[1]) == min(abs(imgxy[1]-ll[1])))[0]

        if xind in [0, len(output[0,:])-1] or yind in [0, len(output[:,0])-1]:
            logger.warn('Latitude and/or longitude for {} in [validate] '
                        'point_values_csv are outside of the model domain, '
                        'exiting point_values'.format(sta,filename))
            return

        pixel_swe.loc[i,'station'] = sta
        pixel_swe.loc[i,'x'] = lat
        pixel_swe.loc[i,'y'] = lon
        pixel_swe.loc[i,'model x'] = int(xind)
        pixel_swe.loc[i,'model y'] = int(yind)

        if heading is not None:
            pixel_swe.loc[i,heading] = df.loc[i,heading]

        # Get the median of the nine closest pixels
        val = np.array([])
        for n in range(-1,1):
            for m in range(-1,1):
                val = np.append(val,output[yind+n,xind+m])
                if n == 0 and m == 0:
                    pixel_swe.loc[i,header] = np.round(float(output[yind+n,xind+m]),decimals = 2)

        pixel_swe.loc[i,'9 pixel median'] = np.round(np.nanmedian(val),decimals = 2)

    pixel_swe.to_csv(filename)

    ###########################################################################
    #   figures
    ###########################################################################

    if heading is not None:

        width = settings[0]
        height = settings[1]
        dpi = settings[2]
        nrows = settings[3]
        ncols = settings[4]
        font_small = settings[5]
        font_medium = settings[6]
        npix = settings[7]
        ss = settings[8]
        levels = settings[9]
        annot_x1 = settings[10]
        annot_y1 = settings[11]
        annot_x2 = settings[12]
        annot_y2 = settings[13]

        sns.set_style('white')
        plt.close(0)
        f, a = plt.subplots(num=0,figsize=(int(width),int(height)),
                            dpi=dpi,nrows=nrows,ncols=ncols)
        a = a.flatten()

        plt.close(1)
        f1, a1 = plt.subplots(num=1,figsize=(int(width),int(height)),
                            dpi=dpi,nrows=nrows,ncols=ncols)
        a1 = a1.flatten()

        plt.close(2)
        f2 = plt.figure(num=2,figsize=(width,height-2),dpi=dpi)
        a2 = plt.gca()

        plt.close(3)
        f3 = plt.figure(num=3,figsize=(width,height-2),dpi=400)
        a3 = plt.gca()

        lvls = list(np.arange(np.min(dem.flatten()), np.max(dem.flatten()), 500))

        snow_map = cmocean.cm.haline_r

        # get min/max for all sub-domains
        pixel_swe = pd.DataFrame(pixel_swe)
        veg = []
        for idx in pixel_swe.index:
            name = pixel_swe.loc[idx,'station']
            ix = int(pixel_swe.loc[idx]['model x'])
            iy = int(pixel_swe.loc[idx]['model y'])

            veg_sub = veg_type[(iy-npix):(iy+npix+1),(ix-npix):(ix+npix+1)].flatten()
            veg = np.append(veg_sub, veg)

            if idx == 0:
                g_min = np.nanmin(output[(iy-npix):(iy+npix),(ix-npix):(ix+npix)])
                g_max = np.nanmax(output[(iy-npix):(iy+npix),(ix-npix):(ix+npix)])

            else:
                vmin = np.nanmin(output[(iy-npix):(iy+npix),(ix-npix):(ix+npix)])
                vmax = np.nanmax(output[(iy-npix):(iy+npix),(ix-npix):(ix+npix)])

                if vmin < g_min:
                    g_min = vmin

                if vmax > g_max:
                    g_max = vmax

        ymin = copy.deepcopy(g_min)
        # g_min = 0
        veg = np.unique(veg)
        vl = list(veg)
        norm = matplotlib.colors.BoundaryNorm(vl,len(vl))

        h3 = a3.imshow(veg_type, cmap ='Vega20', norm = norm, alpha = 0.75)
        a3.contour(dem, colors = 'k', levels=lvls, linewidths = 0.15)

        place = np.linspace(g_min,g_max,256)

        sep = 0.05
        wid = 1/len(pixel_swe.station.unique())-sep
        widths = np.arange((-1 + wid), (1 - wid), wid)

        for idx, v in enumerate(pixel_swe.station.unique()):
            mflag = False

            if idx < len(pixel_swe.station.unique()) + 1:

                # this plots repeating values
                if len(pixel_swe[pixel_swe['station'] == v]['station'].values) > 1:
                    mflag = True
                    vstr = mstr = ''

                    for idx3 in range(0,len(pixel_swe[pixel_swe['station'] == v]['station'].values)):
                        course = pixel_swe.loc[idx + idx3][heading]
                        ix = int(pixel_swe[pixel_swe['station'] == v]['model x'].values[idx3])
                        iy = int(pixel_swe[pixel_swe['station'] == v]['model y'].values[idx3])
                        ixs = int(np.where(abs(place-course) == min(abs(place-course)))[0])
                        color = snow_map(ixs)

                        if idx3 == 0:
                            vstr = '{}'.format(round(course,1))
                            mstr = '{}'.format(output[iy,ix].round(1))
                        else:
                            vstr = vstr + ', {}'.format(round(course,1))
                            mstr = mstr + ', {}'.format(output[iy,ix].round(1))

                        a[idx].scatter(x = ix, y = iy, marker = '^', s=ss, c = color,
                                       edgecolors = 'k', linewidths = 0.5)

                    cstr = 'validation: {}\nmodel: {}'.format(vstr, mstr)
                    a[idx].text(annot_x1, annot_y1, cstr, horizontalalignment='left',
                                transform=a[idx].transAxes,fontsize = font_small - 1,
                                color = 'w')

                ix = int(pixel_swe[pixel_swe['station'] == v]['model x'].values[0])
                iy = int(pixel_swe[pixel_swe['station'] == v]['model y'].values[0])
                swe = pixel_swe[pixel_swe['station'] == v][header].values[0]
                course = pixel_swe[pixel_swe['station'] == v][heading].values[0]

                s = dem[(iy-npix):(iy+npix+1),(ix-npix):(ix+npix+1)]
                snow_sec = output[(iy-npix):(iy+npix+1),(ix-npix):(ix+npix+1)]

                l1 = [int(x)*snow_sec.shape[0]+npix-1 for x in range(npix-1,npix+2)] + \
                     [int(x)*snow_sec.shape[0]+npix for x in range(npix-1,npix+2)] + \
                     [int(x)*snow_sec.shape[0]+npix+1 for x in range(npix-1,npix+2)]
                zo = 1
                if idx == 0:
                    ml = 'model'
                    vl = 'validation'
                    sl = r'$\pm${} pixels'.format(npix)
                    bl1 = r'$\pm$1 pixel'.format(npix)

                else:
                    ml = '__nolabel__'
                    vl = '__nolabel__'
                    sl = '__nolabel__'
                    bl1 = '__nolabel__'

                hv = a2.plot(idx, course, 'bP', markersize = 5, label = vl, zorder = 4)
                hs = a2.plot(idx, swe, 'rX', markersize = 5, label = ml, zorder = 3)
                b1 = a2.boxplot(snow_sec.flatten(), positions = [idx - 0.22],widths = 0.18, showfliers = False)
                b2 = a2.boxplot(snow_sec.flatten()[l1], positions = [idx + 0.22], widths = 0.18, showfliers = False)
                a2.plot(0,ymin,'dimgray', label = sl)
                a2.plot(0,ymin,'forestgreen', label = bl1)
                if idx == 0:
                    astr = 'val-mod [in], %:\n{}\n{}%'.format(round((course-swe),1),int((swe/course)*100))
                else:
                    astr = '{}\n{}%'.format(round((course-swe),1),int((swe/course)*100))

                a2.annotate(astr, (idx - 0.25, ymin - 2 ), fontsize = font_small, color = 'k')

                for prop in ['boxes','whiskers','fliers','caps']:
                    plt.setp(b1[prop], color = 'dimgray')
                    plt.setp(b2[prop], color = 'forestgreen')

                lvls = list(np.arange(np.min(s.flatten()), np.max(s.flatten()), levels))
                ixs = int(np.where(abs(place-course) == min(abs(place-course)))[0])

                color = snow_map(ixs)

                a[idx].contour(dem, colors = 'k', levels=lvls, linewidths = 0.25)
                h = a[idx].imshow(output, cmap = snow_map, clim = (g_min, g_max))

                if not mflag:
                    a[idx].scatter(x = ix, y = iy, marker = '^', s=ss, c = color,
                                   edgecolors = 'k', linewidths = 0.5)
                    cstr = 'validation: {}\nmodel: {}'.format(round(course,1),
                                                             output[iy,ix].round(1))
                    a[idx].text(annot_x1, annot_y1,cstr,horizontalalignment='left',
                                transform=a[idx].transAxes,fontsize = font_small,
                                color = 'w')

                a[idx].get_xaxis().set_ticks([])
                a[idx].get_yaxis().set_ticks([])
                # a[idx].set_ylim((iy - (npix + 0.5), iy + npix + 0.5))
                # a[idx].set_xlim((ix - (npix + 0.5), ix + npix + 0.5))

                a[idx].set_ylim((iy + npix + 0.5),(iy - (npix + 0.5)))
                a[idx].set_xlim((ix - (npix + 0.5), ix + npix + 0.5))

                a[idx].set_title(v, fontsize = font_small)

                # legend of sorts
                veg_subp = veg_type[(iy-npix):(iy+npix+1),(ix-npix):(ix+npix+1)]
                vegmap = plt.cm.get_cmap('Vega20')

                a1[idx].contour(dem, colors = 'k', levels=lvls, linewidths = 0.25)
                h1 = a1[idx].imshow(veg_type, cmap ='Vega20', norm = norm)
                a1[idx].scatter(x = ix, y = iy, marker = '^', s=ss, c = 'k',
                               edgecolors = 'k', linewidths = 0.5)
                a1[idx].get_xaxis().set_ticks([])
                a1[idx].get_yaxis().set_ticks([])

                a1[idx].set_ylim((iy + npix + 0.5),(iy - (npix + 0.5)))
                a1[idx].set_xlim((ix - (npix + 0.5), ix + npix + 0.5))
                a1[idx].set_title(v, fontsize = font_small)

                a3.scatter(x = ix, y = iy, marker = 's', s=10, color="None",
                           edgecolors = 'k', linewidths = 0.5)
                a3.annotate(v, (ix+5, iy-5), fontsize = font_small-1, color = 'k')

                for vs in np.unique(veg_subp):
                    ixf = np.where(veg_subp == vs)
                    fx = ixf[0][0]
                    fy = ixf[1][0]
                    nx = ix + fy - npix
                    ny = iy + fx - npix
                    a1[idx].scatter(x = nx, y = ny, marker = 'X', s=3, c = 'k',
                                   edgecolors = 'k', linewidths = 0.5)
                    a1[idx].annotate(str(vs), (nx, ny), fontsize = font_small - 3, color = 'w')

        # tack on another subplot for legend and colorbar
        a[idx+1].imshow(output, cmap = snow_map, alpha = 0)
        a[idx+1].set_title('', fontsize = font_small)
        a[idx+1].get_xaxis().set_ticks([])
        a[idx+1].get_yaxis().set_ticks([])
        a[idx+1].axis('off')
        cbaxes = inset_axes(a[idx+1], width="90%", height="10%", loc=8)
        cbar = f.colorbar(h, cax=cbaxes, ticks = [int(g_min + 1), int(((g_max - g_min)/2) + g_min), int(g_max)],
                          orientation='horizontal')
        cbar.ax.tick_params(labelsize=font_medium)
        cbar.set_label(cbarlabel, fontsize = font_medium)
        cbar.ax.xaxis.set_label_position('top')
        cstr = (r'Validation pixel $\pm${} pixels'
              '\nmodel: {}\nvalidation: {}\ncontour interval: {} '
              'ft\npixel width: {} m, {} ft'.format(npix, model_date, course_date,
              levels, str(pixel), str(int(pixel*3.28))))
        a[idx+1].text(annot_x2, annot_y2,cstr,horizontalalignment='left',
                    transform=a[idx+1].transAxes,fontsize = font_medium)

        a1[idx+1].imshow(veg_type, cmap = vegmap, clim = (np.min(veg),np.max(veg)), alpha = 0)
        a1[idx+1].set_title('', fontsize = font_small)
        a1[idx+1].get_xaxis().set_ticks([])
        a1[idx+1].get_yaxis().set_ticks([])
        a1[idx+1].axis('off')
        cstr = ('model: {}\nvalidation: {}\ncontour interval: {} '
              'ft\npixel width: {} m, {} ft'.format(model_date, course_date,
              levels, str(pixel), str(int(pixel*3.28))))
        a1[idx+1].text(annot_x2, annot_y2,cstr,horizontalalignment='left',
                    transform=a1[idx+1].transAxes,fontsize = font_medium)

        for n in range(idx+2,nrows*ncols):
            f.delaxes(a[n])

        for n in range(idx+2,nrows*ncols):
            f1.delaxes(a1[n])

        a2.set_ylabel(cbarlabel)
        a2.set_xticks(list(range(0,len(pixel_swe.station.unique()))))
        a2.set_xticklabels(pixel_swe.station.unique(), rotation = 90, fontsize = font_small - 1)
        a2.set_xlim((-1,len(pixel_swe.station.unique())+1))
        a2.legend(loc='upper left')
        a2.grid(linewidth = 0.25)
        a2.set_title('Validation and Model Pixel Values, {}'.format(model_date))

        a3.get_xaxis().set_ticks([])
        a3.get_yaxis().set_ticks([])
        divider3 = make_axes_locatable(a3)
        cax3 = divider3.append_axes("right", size="4%", pad=0.2)
        cbar3 = plt.colorbar(h3, cax = cax3, ticks = list(veg))
        cbar3.ax.tick_params()
        cbar3.set_label('vegetation type classification')

        f.tight_layout()
        f1.tight_layout()
        f2.tight_layout()
        f3.tight_layout()

        fig_name_short = 'validation_list_{}'.format(value)
        fig_name = '{}{}.png'.format(figs_path,fig_name_short)
        logger.info(' saving {}'.format(fig_name))
        snowav.framework.figures.save_fig(f2, fig_name)

        fig_name_short = 'validation_map_{}'.format(value)
        fig_name = '{}{}.png'.format(figs_path,fig_name_short)
        logger.info(' saving {}'.format(fig_name))
        snowav.framework.figures.save_fig(f, fig_name)

        fig_name_short = 'validation_veg_map'
        fig_name = '{}{}.png'.format(figs_path,fig_name_short)
        logger.info(' saving {}'.format(fig_name))
        snowav.framework.figures.save_fig(f1, fig_name)

        fig_name_short = 'validation_locations'
        fig_name = '{}{}.png'.format(figs_path,fig_name_short)
        logger.info(' saving {}'.format(fig_name))
        snowav.framework.figures.save_fig(f3, fig_name)

        return
